function MakeHUD(parent, scene) {

  function makeLine(){

    var lineMaterial = new THREE.LineBasicMaterial({
      color: 0x00ff00, opacity: 0.3, transparent: true
    });

    var lineGeometry = new THREE.Geometry();


    return new THREE.LineSegments(lineGeometry, lineMaterial, THREE.LinePieces);
  }


  var lineMarker = makeLine();
  var relativeMarker = makeLine();

  lineMarker.name = "HUD line marker";
  relativeMarker.name = "HUD relative marker";

  function makeText(deg, callback) {

    var textMat = new THREE.MeshBasicMaterial({
     color: 0x00ff00, opacity: 0.3, transparent: true
    });
    var loader = new THREE.FontLoader();
    loader.load( 'droid_sans_regular.typeface.js', function ( font ) {
      var geometry = new THREE.TextGeometry(deg.toString(), {
        text: deg, font: font, size: 0.50, height: 0
      });
      geometry.center();

      var mesh = new THREE.Mesh(geometry, textMat);
      callback(mesh);
    });
  }

  var RADIUS = 20;

  var quat = new THREE.Quaternion();
  for (var theta = 0; theta < 360; theta += 10){
    var angle = -theta * Math.PI / 180;
    quat.setFromAxisAngle(new THREE.Vector3(1, 0, 0), angle);

    var isHorizon = (theta == 0 || theta == 180);
    var isPole = (theta == 90 || theta == 270);
    var w = isHorizon ? 8 : isPole ? 0 : 3;
    var v1 = new THREE.Vector3(-w, 0, RADIUS);
    var v2 = new THREE.Vector3(w, 0, RADIUS);

    v1.applyQuaternion(quat);
    v2.applyQuaternion(quat);

    var quat1 = new THREE.Quaternion().setFromAxisAngle(new THREE.Vector3(1, 0, 0), angle)
    .multiply(
      new THREE.Quaternion().setFromAxisAngle(new THREE.Vector3(0, 1, 0), Math.PI)
    );

    var theta1 = theta;
    if (theta1 < 90)
      theta1 -= 0;
    else if (theta1 < 180)
      theta1 = 180 - theta1;
    else if (theta1 < 270)
      theta1 = -(theta1 - 180);
    else
      theta1 = -(360 - theta1);

    if (!isHorizon && !isPole) (function(v1, v2, quat1) {
    makeText(theta1, function(lDeg) {
      lDeg.position.copy(v1);
      lDeg.quaternion.copy(quat1);
      lineMarker.add(lDeg);
    });

    makeText(theta1, function(rDeg) {
      rDeg.position.copy(v2);
      rDeg.quaternion.copy(quat1);
      lineMarker.add(rDeg);
    });
    })(v1, v2, quat1);


    lineMarker.geometry.vertices.push(v1, v2);
  }

  lineMarker.geometry.vertices.push(
    new THREE.Vector3(-0.25, RADIUS, 0),
    new THREE.Vector3(0.25, RADIUS, 0),
    new THREE.Vector3(0,RADIUS,-0.25),
    new THREE.Vector3(0,RADIUS,0.25),

    new THREE.Vector3(-0.25, -RADIUS, 0),
    new THREE.Vector3(0.25, -RADIUS, 0),
    new THREE.Vector3(0,-RADIUS,-0.25),
    new THREE.Vector3(0,-RADIUS,0.25)
    );

  relativeMarker.geometry.vertices.push(
    new THREE.Vector3(0.25, 0, -RADIUS),
    new THREE.Vector3(-0.25,0,-RADIUS),
    new THREE.Vector3(0,0.25,-RADIUS),
    new THREE.Vector3(0,-0.25,-RADIUS)
    );

  parent.add(lineMarker);
  parent.add(relativeMarker);

  function update(rotation, direction) {
    var cameraDir = V3().copy(direction);
    cameraDir.y = 0;
    lineMarker.lookAt(cameraDir);
    relativeMarker.lookAt(V3().copy(direction).negate());
  }

  return {lineMarker: lineMarker, relativeMarker: relativeMarker, update: update};

}
