/// <reference path="jquery.d.ts" />
/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />
// Labels.ts
function makeText(params) {
    if (params === void 0) { params = null; }
    var domElement = $('<div class="text"></div>');
    domElement.css("position", "absolute").css("color", "white")
        .css("font-family", "sans-serif");
    if (params)
        domElement.css(params);
    $('body').append(domElement);
    return domElement;
}
var Label = (function () {
    function Label(text) {
        if (text === void 0) { text = makeText(); }
        this.text = text;
        this._newText = "";
        this._newX = 0;
        this._newY = 0;
        this.text.css({
            "-moz-user-select": "none",
            "-webkit-user-select": "none",
            "user-select": "none",
            "pointer-events": "none",
            "font-size": "8pt",
            "font-weight": "100",
            color: "#ffaa00",
            opacity: "0.7",
            "text-align": "center",
            top: "-1000px",
            left: "-1000px"
        });
    }
    Label.prototype.setText = function (text1) {
        this._newText = text1;
    };
    Label.prototype.reposition = function (obj, dx, dy) {
        if (dx === void 0) { dx = 0; }
        if (dy === void 0) { dy = 10; }
        var xyz = LabelManager.instance.toScreenPosition(obj);
        if (xyz === null) {
            this._newX = -100;
            this._newY = -100;
        }
        else {
            this._newX = xyz.x + dx;
            this._newY = xyz.y + dy;
        }
    };
    Label.prototype.update = function () {
        if (this._newX || this._newY)
            this.text.css({ top: this._newY, left: this._newX });
        if (this._newText)
            this.text.text(this._newText);
        this._newX = null;
        this._newY = null;
        this._newText = null;
    };
    Label.prototype.remove = function () {
        LabelManager.instance.removeLabel(this);
    };
    return Label;
}());
var LabelManager = (function () {
    function LabelManager(renderer, camera) {
        var _this = this;
        this.renderer = renderer;
        this.camera = camera;
        this._labels = [];
        if (LabelManager.instance != null) {
            throw new Error("LabelManager is already instantiated.");
        }
        LabelManager.instance = this;
        setInterval(function () {
            _this._labels.forEach(function (l) { return l.update(); });
        }, 1000 / 60);
    }
    LabelManager.prototype.createLabel = function () {
        var newLabel = new Label();
        this._labels.push(newLabel);
        return newLabel;
    };
    LabelManager.prototype.removeLabel = function (l) {
        if (l.text === null)
            console.warn("Label already removed. ", l);
        l.text.remove();
        l.text = null;
        _.remove(this._labels, function (l1) { return l1 === l; });
    };
    // http://stackoverflow.com/a/27410603/303939
    // Convert camera position to screen position
    LabelManager.prototype.toScreenPosition = function (obj) {
        var camera = this.camera, renderer = this.renderer;
        var vector = new THREE.Vector3();
        var widthHalf = 0.5 * renderer.context.canvas.width;
        var heightHalf = 0.5 * renderer.context.canvas.height;
        obj.updateMatrixWorld();
        vector.setFromMatrixPosition(obj.matrixWorld);
        vector.project(camera);
        var cameraForward = camera.getWorldDirection();
        var displacement = new THREE.Vector3().subVectors(obj.getWorldPosition(), camera.getWorldPosition());
        if (displacement.dot(cameraForward) < 0)
            return null;
        vector.x = (vector.x * widthHalf) + widthHalf;
        vector.y = -(vector.y * heightHalf) + heightHalf;
        return {
            x: vector.x,
            y: vector.y,
            z: vector.z
        };
    };
    return LabelManager;
}());
//# sourceMappingURL=Labels.js.map