/// <reference path="jquery.d.ts" />
/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />

// Labels.ts

function makeText(params = null as any) {
  var domElement = $('<div class="text"></div>');
  domElement.css("position", "absolute").css("color", "white")
            .css("font-family", "sans-serif");

  if (params) domElement.css(params);

  $('body').append(domElement);
  return domElement;
}

class Label {
  private _newText = "";
  private _newX = 0;
  private _newY = 0;

  constructor(public text = makeText()) {
    this.text.css({
      "-moz-user-select": "none",
      "-webkit-user-select": "none",
      "user-select": "none",

      "pointer-events": "none",
      "font-size": "8pt",
      "font-weight": "100",
      color: "#ffaa00",
      opacity: "0.7",
      "text-align": "center",
      top: "-1000px",
      left: "-1000px"
    });
  }

  setText(text1 : string) {
    this._newText = text1;
  }

  reposition(obj, dx = 0, dy = 10) {
    var xyz = LabelManager.instance.toScreenPosition(obj);
    if (xyz === null) {
      this._newX = -100; this._newY = -100;
      //this.text.css({top: -100, left: -100});
    } else {
      this._newX = xyz.x + dx;
      this._newY = xyz.y + dy;
    }
  }

  update() {
    if (this._newX || this._newY)
      this.text.css({top: this._newY, left: this._newX});

    if (this._newText)
      this.text.text(this._newText);

    this._newX = null; this._newY = null; this._newText = null;
  }

  remove() {
    LabelManager.instance.removeLabel(this);
  }
}

class LabelManager {
  static instance : LabelManager;

  private _labels : Label[] = [];

  constructor(
    public renderer : THREE.WebGLRenderer,
    public camera : THREE.Camera
  ) {
    if (LabelManager.instance != null) {
      throw new Error("LabelManager is already instantiated.");
    }
    LabelManager.instance = this;

    setInterval(() => {
      this._labels.forEach((l) => l.update());
    }, 1000 / 60);
  }

  createLabel() : Label {
    var newLabel = new Label();
    this._labels.push(newLabel);
    return newLabel;
  }

  removeLabel(l : Label) {
    if (l.text === null) console.warn("Label already removed. ", l);
    l.text.remove();
    l.text = null;
    _.remove(this._labels, (l1) => l1 === l);
  }


  // http://stackoverflow.com/a/27410603/303939
  // Convert camera position to screen position
  toScreenPosition(obj : any) : {x : number, y : number, z : number} {
    var camera = this.camera, renderer = this.renderer;
    var vector = new THREE.Vector3();

    var widthHalf = 0.5 * renderer.context.canvas.width;
    var heightHalf = 0.5 * renderer.context.canvas.height;

    obj.updateMatrixWorld();
    vector.setFromMatrixPosition(obj.matrixWorld);
    vector.project(camera);

    var cameraForward = camera.getWorldDirection();
    var displacement = new THREE.Vector3().subVectors(obj.getWorldPosition(), camera.getWorldPosition());
    if (displacement.dot(cameraForward) < 0)
      return null;

    vector.x = ( vector.x * widthHalf ) + widthHalf;
    vector.y = - ( vector.y * heightHalf ) + heightHalf;

    return {
      x: vector.x,
      y: vector.y,
      z: vector.z
    };

  }
}

