/// <reference path="jQuery.d.ts" />
/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />
/// <reference path="Utils.ts" />
/// <reference path="OrbitalMechanics.ts" />
/// <reference path="Rings.ts" />
/// <reference path="SOITree.ts" />
/// <reference path="SolarSystem.ts" />
/// <reference path="Labels.ts" />
/// <reference path="Body.ts" />
// Objectives.ts
var Objectives = (function () {
    function Objectives(view, label) {
        this.view = view;
        this.label = label;
        this.score = 0;
        // distance you need to travel
        this.targetDistance = 0;
        this.time = 0;
        this.deadline = 0;
        this.target = null;
        this.distance = 0;
        this.view = view;
        this.reset();
        this.start();
    }
    Objectives.prototype.getTarget = function () {
        var res = _.sample(this.view.tree.root.children);
        var name = res.data.name;
        if (name == "Sun" || name == "Planet Nine")
            return this.getTarget();
        else
            return res;
    };
    Objectives.prototype.start = function () {
        this.setNextTarget(this.time);
    };
    Objectives.prototype.setNextTarget = function (t) {
        this.deadline = t + this.getTimeLimit();
        this.targetDistance = this.getTargetDistance();
        this.target = this.getTarget();
    };
    Objectives.prototype.getTimeLimit = function () {
        var score = this.score;
        if (score < 3)
            return 600;
        else if (score < 10)
            return 100;
        else if (score < 50)
            return 60;
        else
            return 40;
    };
    Objectives.prototype.getTargetDistance = function () {
        var score = this.score;
        if (score < 3)
            return 0.15 * au;
        else if (score < 5)
            return 50e3 * km;
        else if (score < 14)
            return 3000 * km;
        else if (score < 20)
            return 200 * km;
        else
            return 50 * km;
    };
    Objectives.prototype.reset = function () {
        this.score = 0;
        this.target = null;
        this.targetDistance = 0;
        this.deadline = 0;
    };
    Objectives.prototype.onCollision = function () {
        console.log("collision");
        this.gameOver(false);
    };
    Objectives.prototype.update = function (t) {
        var _this = this;
        this.time = t;
        if (!this.target)
            return;
        if (this.target)
            this.distance = this.view.getRelativePositionTo(this.target).length() -
                this.target.data.radius;
        if (this.distance <= 10 * km) {
            this.onCollision();
        }
        if (this.distance <= this.targetDistance) {
            this.label.text("Good job!");
            this.score++;
            this.target = null;
            setTimeout(function () {
                _this.start();
            }, 1000);
            return;
        }
        if (this.time >= this.deadline) {
            this.gameOver(true);
        }
        this.label.html(this.getDescription());
    };
    Objectives.prototype.gameOver = function (reason) {
        console.log("Game over!");
        var reasonText = (!reason ? "You crashed your ship." : "You ran out of time.");
        alert("Game over! " + reasonText + "\nYour score is " + this.score + ".");
        window.location.reload();
        this.reset();
    };
    Objectives.prototype.getDescription = function () {
        return ("Objective: Visit <strong>" + this.target.data.name + "</strong> " +
            "within " + formatDistance(this.targetDistance, 1) + "<br />" +
            "Current Distance: " + formatDistance(this.distance, 5) + " <br />" +
            "Score: " + this.score + "<br />" +
            "Time Left: " + (this.deadline - this.time).toFixed(0) + "s <br />");
    };
    return Objectives;
}());
//# sourceMappingURL=Objectives.js.map