/// <reference path="three.d.ts" />
/// <reference path="Utils.ts" />
/// <reference path="SolarSystem.ts" />

// Adapted from P2 code.

function MakeRingMesh(rings : RingsData) {
  var thetaSegments = 128, phiSegments = 4;
  var ringGeom = new THREE.RingGeometry(
    rings.innerRadius,
    rings.outerRadius,
    thetaSegments, phiSegments
  );

  var uvs = [], faceVertexUvs = [], i, o;

  // Create UV mapping for radial texture
  //   http://stackoverflow.com/questions/24634971/how-can-i-apply-a-radial-texture-to-a-ring-with-three-js-r67
  // The following for loops are adapted from:
  //   https://github.com/mrdoob/three.js/blob/master/src/extras/geometries/RingGeometry.js
  for (i = 0; i < phiSegments + 1; i++) {
    for (o = 0; o < thetaSegments + 1; o++) {
      uvs.push(new THREE.Vector2(o / thetaSegments, i / phiSegments));
      //uvs.push(new THREE.Vector2(i / phiSegments, o / thetaSegments));
    }
  }

  for (i = 0; i < phiSegments; i++) {
    var thetaSegment = i * ( thetaSegments + 1 );
    for (o = 0; o < thetaSegments; o++) {
      var segment = o + thetaSegment;

      var v1 = segment;
      var v2 = segment + thetaSegments + 1;
      var v3 = segment + thetaSegments + 2;
      faceVertexUvs.push([uvs[v1].clone(), uvs[v2].clone(), uvs[v3].clone()]);

      v1 = segment;
      v2 = segment + thetaSegments + 2;
      v3 = segment + 1;

      faceVertexUvs.push([uvs[v1].clone(), uvs[v2].clone(), uvs[v3].clone()]);
    }
  }

  ringGeom.faceVertexUvs[0] = faceVertexUvs;
  ringGeom.uvsNeedUpdate = true;

  var ringMat = rings.material;
  if (!ringMat) {
    ringMat = new THREE.MeshPhongMaterial({
      specular: 0,
      shininess: 0,
      color: rings.color || 0x444444,
      // emission is required because the rings
      // scatter light to all directions equally
      emissive: rings.color || 0x444444,
      side: THREE.DoubleSide,
      map: rings.texture || null,
      lightMap: rings.map || null,
      alphaMap: rings.alphaMap || null,
      transparent: true,
      //blending: THREE.AdditiveBlending
    });
  }

  var ringMesh = new THREE.Mesh(ringGeom, ringMat);
  ringMesh.rotateOnAxis(new THREE.Vector3(1, 0, 0), -90 * deg);
  ringMesh.receiveShadow = ringMesh.castShadow = true;

  return ringMesh;
}

