/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />
/// <reference path="Utils.d.ts" />
/**
 * Sphere of Influence Tree:
 *  Divides the world into a hierarchy of nodes, each node has a sphere of
 *  influence radius. When a point is inside a tree, the closest SOI tree node
 *  is determined.
 */
declare class SOITree {
    root: SOINode;
    time: number;
    constructor(rootRadius: number, rootData?: any);
}
declare class SOINode {
    soiTree: SOITree;
    parent: SOINode;
    relPos: Vector3;
    radius: number;
    children: SOINode[];
    /** Data associated to this SOINode. */
    data: any;
    /** Storage for other classes to modify. */
    storage: any;
    constructor(soiTree: SOITree, parent: SOINode, relPos: Vector3, radius: number, children: SOINode[], 
        /** Data associated to this SOINode. */
        data?: any, 
        /** Storage for other classes to modify. */
        storage?: any);
    getParentPosition(pos: Vector3, into?: THREE.Vector3): THREE.Vector3;
    getGlobalPosition(into?: THREE.Vector3): Vector3;
    addChild(relPos: Vector3, radius: number, data?: any): SOINode;
    contains(otherRelPos: Vector3): boolean;
    containsByParentPos(otherRelPos: Vector3): boolean;
    /**
     * Choose a new child by finding a closer object.
     * parentPos is the position relative to parent. For example, if parent is
     * Earth, then parentPos is relative to Earth and the Moon is chosen if the
     * ship is closer to Moon than Earth.
     */
    findChildByParentPos(parentPos: Vector3): SOINode;
    getRelativePositionList(): [{
        pos: Vector3;
        node: SOINode;
        name: string;
    }];
    relativePositionTo(node: SOINode): Vector3;
    sqDistanceTo(node: SOINode): number;
    distanceTo(node: SOINode): number;
}
declare class SOICursor {
    soiTree: SOITree;
    cursor: SOINode;
    localPos: THREE.Vector3;
    private _onMoveToParent;
    private _onMoveToChild;
    constructor(soiTree: SOITree, cursor: SOINode, localPos?: THREE.Vector3);
    move(deltaPos: Vector3): number;
    getGlobalPosition(into?: THREE.Vector3): Vector3;
    private moveToParent();
    private moveToChild(node);
    onMoveToParent(f: any): void;
    onMoveToChild(f: any): void;
}
