/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />
/// <reference path="Utils.ts" />
/// <reference path="OrbitalMechanics.ts" />
;
// Solar system data taken from our P2 code
function getSolarSystemData() {
    // Planet data from Wolfram|Alpha
    return {
        "Mercury": {
            color: 0x444444,
            radius: 2439.7 * km,
            rotation: {
                period: 58.6462 * day,
                tilt: 0.01 * deg
            },
            orbit: {
                semiMajorAxis: 0.38709893 * au,
                eccentricity: 0.20563069,
                period: 87.96926 * day,
                phase: 60 * deg,
                inclination: 7.00487 * deg,
                argumentOfPeriapsis: 29.1248 * deg,
                longitudeOfAscendingNode: 48.33167 * deg
            }
        },
        "Venus": {
            color: 0xffaa44,
            radius: 6051.9 * km,
            rotation: {
                period: -243.018 * day,
                tilt: 177.36 * deg
            },
            orbit: {
                semiMajorAxis: 0.72333199 * au,
                eccentricity: 0.00677373,
                period: -224.7008 * day,
                phase: 200 * deg,
                inclination: 3.39471 * deg,
                argumentOfPeriapsis: 54.85229 * deg,
                longitudeOfAscendingNode: 76.68069 * deg
            }
        },
        "Earth": {
            color: 0x0033ff,
            material: new THREE.MeshPhongMaterial({
                color: 0xddeeff,
                map: new THREE.TextureLoader().load('Textures/Earth4096.png'),
                normalMap: new THREE.TextureLoader().load('Textures/EarthNormal.png'),
                bumpMap: new THREE.TextureLoader().load('Textures/Earth.png'),
                specularMap: new THREE.TextureLoader().load('Textures/EarthSpec.png'),
                shininess: 25,
                specular: 0x222222
            }),
            radius: 6378.137 * km,
            rotation: {
                period: 23.934472 * hour,
                phase: 10 * deg,
                tilt: 23.4 * deg
            },
            orbit: {
                semiMajorAxis: 1.00000011 * au,
                eccentricity: 0.01671022,
                period: 1 * year,
                phase: 160 * deg
            },
            distanceScale: 1e2
        },
        "Moon": {
            radius: 1737.5 * km,
            color: 0x888888,
            parent: "Earth",
            material: new THREE.MeshPhongMaterial({
                color: 0xeeeeee,
                map: new THREE.TextureLoader().load('Textures/Moon.png'),
                normalMap: new THREE.TextureLoader().load('Textures/MoonNormal.png'),
                //bumpScale: 0.2,
                shininess: 10,
                specular: 0x111111
            }),
            rotation: {
                period: 27.322 * day,
                phase: 170 * deg
            },
            orbit: {
                semiMajorAxis: 284400 * km,
                eccentricity: 0.0554,
                period: 27.322 * day,
                phase: 0,
                inclination: 5.16 * deg,
                argumentOfPeriapsis: 318.15 * deg,
                longitudeOfAscendingNode: 125.08 * deg
            }
        },
        "Mars": {
            color: 0xff7733,
            radius: 3396.19 * km,
            rotation: {
                period: 24.6229622 * hour,
                tilt: 25.19 * deg
            },
            orbit: {
                semiMajorAxis: 1.52366231 * au,
                eccentricity: 0.09341233,
                period: 1.88 * year,
                phase: 0 * deg,
                inclination: 1.85061 * deg,
                argumentOfPeriapsis: 286.4623 * deg,
                longitudeOfAscendingNode: 49.57854 * deg
            }
        },
        "Jupiter": {
            radius: 71492 * km,
            color: 0xffcc33,
            material: new THREE.MeshPhongMaterial({
                color: 0xaaaaaa,
                map: new THREE.TextureLoader().load('Textures/Jupiter.png'),
                shininess: 10,
                specular: 0x050505
            }),
            rotation: {
                period: 9.925 * hour,
                tilt: 3.13 * deg,
                phase: 100 * deg
            },
            orbit: {
                semiMajorAxis: 5.20336301 * au,
                eccentricity: 0.04839266,
                period: 11.862615 * year,
                phase: 0 * deg,
                inclination: 1.3053 * deg,
                argumentOfPeriapsis: -85.8023 * deg,
                longitudeOfAscendingNode: 100.55615 * deg
            },
            distanceScale: 1e2
        },
        "Io": {
            radius: 1821.6 * km,
            color: 0xeedd22,
            parent: "Jupiter",
            rotation: {
                period: 42.46 * hour,
                tilt: 0 * deg,
                phase: 100 * deg
            },
            orbit: {
                semiMajorAxis: 421700 * km,
                eccentricity: 0.0041,
                period: 42.46 * hour,
                phase: 60 * deg,
                inclination: 0.0036 * deg,
                argumentOfPeriapsis: 84.129 * deg,
                longitudeOfAscendingNode: 43.977 * deg
            }
        },
        "Europa": {
            radius: 1560.8 * km,
            color: 0x884400,
            parent: "Jupiter",
            rotation: {
                period: 85.22 * hour,
                tilt: 0.016 * deg,
                phase: 12 * deg
            },
            orbit: {
                semiMajorAxis: 671100 * km,
                eccentricity: 0.0094,
                period: 85.22 * hour,
                phase: 140 * deg,
                inclination: 0.466 * deg,
                argumentOfPeriapsis: 88.97 * deg,
                longitudeOfAscendingNode: 219.106 * deg
            }
        },
        "Ganymede": {
            radius: 2631.2 * km,
            color: 0x664400,
            parent: "Jupiter",
            rotation: {
                period: 7.155 * day,
                tilt: 0.068 * deg,
                phase: 10 * deg
            },
            orbit: {
                semiMajorAxis: 1.0704e6 * km,
                eccentricity: 0.0013,
                period: 7.155 * day,
                phase: 100 * deg,
                inclination: 0.177 * deg,
                argumentOfPeriapsis: 192.417 * deg,
                longitudeOfAscendingNode: 63.552 * deg
            }
        },
        "Callisto": {
            radius: 2410.3 * km,
            color: 0x664444,
            parent: "Jupiter",
            rotation: {
                period: 16.69 * day,
                tilt: 0.356 * deg,
                phase: 150 * deg
            },
            orbit: {
                semiMajorAxis: 1.8827e6 * km,
                eccentricity: 0.0074,
                period: 16.69 * day,
                phase: 0 * deg,
                inclination: 0.192 * deg,
                argumentOfPeriapsis: 192.417 * deg,
                longitudeOfAscendingNode: 298.848 * deg
            }
        },
        "Saturn": {
            radius: 54364 * km,
            color: 0x666055,
            material: new THREE.MeshPhongMaterial({
                color: 0x666055,
                map: new THREE.TextureLoader().load('Textures/Saturn.png'),
                bumpMap: new THREE.TextureLoader().load('Textures/SaturnBump.png'),
                bumpScale: 0.2,
                shininess: 10,
                specular: 0x111111
            }),
            rotation: {
                period: 10.656 * hour,
                phase: 5 * deg,
                tilt: 26.73 * deg
            },
            rings: {
                innerRadius: 74500 * km,
                outerRadius: 140221 * km,
                color: 0x887766,
                texture: new THREE.TextureLoader().load('Textures/SaturnRings.png'),
                alphaMap: new THREE.TextureLoader().load('Textures/SaturnRingsAlpha.png')
            },
            orbit: {
                semiMajorAxis: 9.53707032 * au,
                eccentricity: 0.0541506,
                period: 29.447498 * year,
                phase: 250 * deg,
                inclination: 2.48446 * deg,
                argumentOfPeriapsis: -21.2831 * deg,
                longitudeOfAscendingNode: 113.71504 * deg
            }
        },
        "Uranus": {
            color: 0xbbeeee,
            radius: 24973 * km,
            orbit: {
                semiMajorAxis: 19.19126393 * au,
                eccentricity: 0.04716771,
                period: -84.016846 * year,
                phase: 20 * deg,
                inclination: 0.76986 * deg,
                argumentOfPeriapsis: 96.73436 * deg,
                longitudeOfAscendingNode: 74.22988 * deg
            },
            rotation: {
                period: 17.24 * hour,
                phase: 130 * deg,
                tilt: 97.77 * deg
            }
        },
        "Neptune": {
            color: 0x77aacc,
            radius: 24341 * km,
            orbit: {
                semiMajorAxis: 30.06896348 * au,
                eccentricity: 0.00858587,
                period: 164.79132 * year,
                phase: 150 * deg,
                inclination: 1.76917 * deg,
                argumentOfPeriapsis: -86.75034 * deg,
                longitudeOfAscendingNode: 131.72169 * deg
            },
            rotation: {
                period: 16.11 * hour,
                phase: 25 * deg,
                tilt: 28.32 * deg
            }
        },
        "Planet Nine": {
            color: 0x114466,
            radius: 26000 * km,
            orbit: {
                semiMajorAxis: 700 * au,
                eccentricity: 0.6,
                period: 20000 * year,
                phase: 50 * deg,
                inclination: 30 * deg,
                argumentOfPeriapsis: 150 * deg,
                longitudeOfAscendingNode: 0 * deg
            },
            rotation: {
                period: 50 * hour,
                phase: 35 * deg,
                tilt: 33 * deg
            }
        }
    };
}
//# sourceMappingURL=SolarSystem.js.map