/// <reference path="lodash.d.ts" />
/// <reference path="three.d.ts" />

// Utils.ts

// Unit definitions
var second = 1, minute = 60, hour = 3600;
var day = 24 * hour, year = 365.25636 * day;
var m = 1, km = 1e3, Mm = 1e6, Gm = 1e9;
var au = 1.496e8 * km;
var speedOfLight = 299792458;
var deg = Math.PI / 180;

type Vector3 = THREE.Vector3;

function V3(x = 0.0, y = 0.0, z = 0.0) {
  return new THREE.Vector3(x, y, z);
}

function quatAxisAngle(axis, angle) {
  return new THREE.Quaternion().setFromAxisAngle(axis, angle);
}

type Color = THREE.Color;

function Col(r = 0.0, g = 0.0, b = 0.0) {
  return new THREE.Color(r, g, b);
}

function HexCol(x) {
  return new THREE.Color(x);
}

function clamp(x, a, b) {
  if (x < a) return a;
  if (x > b) return b;
  return x;
}


/** Adjust a value exponentially. */
function expAdjust(x, k) {
  return Math.exp(Math.log(x) + k);
}

function formatSpeed(x, p) {
  if (x >= 0.1 * speedOfLight)
    return formatNumber(x / speedOfLight, p) + " c";
  else
    return formatDistance(x, p) + "/s";
}

function formatDistance(x, p) {
  if (x > 1e-1 * au) {
    return formatNumber(x / au, p) + " AU";
  } else {
    return formatNumber(x / km, p) + " km";
  }
}

function formatNumber(x, p) {
  if (typeof p === "undefined") p = 3;

  if (x < 1e5)
    return x.toFixed(p);
  else
    return x.toExponential(p);
}

