// Adapted from P2 code.

var windowWidth, windowHeight;
var canvas, scene, camera, renderer;
var fpsStats;
var mouse = new THREE.Vector2();
var keyboard, timer, t0 = 0.0, simTime = 0.0;
var timeAcc = 1;
var ship;
var ambientLight;
var myAudio;

var system, systemView;

var speedLabel, timeLabel, objectLabel, objectiveLabel;

_.assign(THREEx.KeyboardState.ALIAS, {
  comma: 188,
  period: 190,
  minus: 189,
  equal: 187,
  slash: 191,
  leftbracket: 219,
  rightbracket: 221,
  enter: 13
});

$(function() {
  'use strict';

  canvas = $('#canvas')[0];
  scene = new THREE.Scene();
  scene._targets = [];

  camera = new THREE.PerspectiveCamera(60, window.innerWidth / window.innerHeight,
      0.1, 800 * au);
  renderer = new THREE.WebGLRenderer({
    antialias: true,
    // prevent z-fighting
    logarithmicDepthBuffer: true
  });
  renderer.shadowMap.enabled = true;
  renderer.setClearColor(0);
  canvas.appendChild(renderer.domElement);

  new LabelManager(renderer, camera);

  speedLabel = makeText({bottom: 0, right: 0, margin: '0.4em', 'text-align': 'right'});
  speedLabel.text("---");

  timeLabel = makeText({top: 0, right: 0, margin: '0.4em', 'text-align': 'right'});
  timeLabel.text("---");

  objectLabel = makeText({bottom: 0, left: 0, margin: '0.4em', 'text-align': 'left'});
  objectLabel.text("---");

  objectiveLabel = makeText({
    top: '50%', left: 0,
    margin: '0.4em', 'text-align': 'left'
  });
  objectiveLabel.text("");

  // setup stats
  fpsStats = new Stats();
  fpsStats.setMode(0);

  fpsStats.domElement.style.position = 'absolute';
  fpsStats.domElement.style.top = '0px';
  fpsStats.domElement.style.left = '0px';
  document.body.appendChild(fpsStats.domElement);

  function setupResize() {
    // TODO handle retina resolutions
    // https://www.khronos.org/webgl/wiki/HandlingHighDPI
    function resize() {
      windowWidth = window.innerWidth;
      windowHeight = window.innerHeight;
      renderer.setSize(window.innerWidth, window.innerHeight);
    }
    window.addEventListener('resize',resize);
    resize();
  }

  function inputAxis(negKey, posKey, shiftFactor, defaultValue) {
    if (shiftFactor === void 0) shiftFactor = 1e-2;

    if (shiftFactor !== false && shiftFactor !== null) {
      if (keyboard.pressed('shift+' + posKey)) return +shiftFactor;
      if (keyboard.pressed('shift+' + negKey)) return -shiftFactor;
    }

    return keyboard.pressed(posKey) ? 1 : keyboard.pressed(negKey) ? -1
      : defaultValue || 0;
  }

  function setupEvents() {
    keyboard.domElement.addEventListener('keydown', onKeyDown);
    keyboard.domElement.addEventListener('keyup', function(evt) { });

    renderer.domElement.addEventListener('mousedown', onMouseDown, false);
    renderer.domElement.addEventListener('mouseup', function(evt) { }, false);
    renderer.domElement.addEventListener('mousemove', onMouseMove, false);
  }

	function onMouseMove(event) {
    // http://threejs.org/docs/#Reference/Core/Raycaster

		// calculate mouse position in normalized device coordinates
		// (-1 to +1) for both components

		mouse.x = ( event.clientX / window.innerWidth ) * 2 - 1;
		mouse.y = - ( event.clientY / window.innerHeight ) * 2 + 1;
	}

  function onMouseDown(event) {
    ship.onMouseDown();
  }

  function onKeyDown(event) {
    ship.onKeyDown(event);

    if (keyboard.eventMatches(event, 'o')) {
      var visible = !scene._orbitLines.visible;
      scene._orbitLines.traverse(function(c) { c.visible = visible; });
    }

    if (keyboard.eventMatches(event, 'm')){
      if (myAudio.volume > 0)
        myAudio.volume = 0;
      else myAudio.volume = 1;
    }
  }

  function MakeShip(view, camera) {
    var cursor = view.cursor;
    view.moveCursor(V3(1.2 * au, 0.2 * au, 0));

    var ship = new THREE.Group();
    ship.name = "Ship";
    ship.view = view;

    var shipObject = new THREE.Group();
    shipObject.name = "Ship Object";
    ship.add(shipObject);

    var HUD;

    var vel = V3();
    var fwd = V3(), up = V3(), right = V3();
    var back = V3(), down = V3(), left = V3();
    ship.vel = vel;
    ship.fwd = fwd;
    ship.up = up;
    ship.right = right;
    ship.back = back;
    ship.down = down;
    ship.left = left;

    var dVel = V3(), unitR = V3(), velFwd = V3(), velSide = V3();

    HUD = MakeHUD(ship, scene);

    var raycaster = new THREE.Raycaster();
    raycaster.near = 10 * km;

    var selection = null;

    var particleSystem = createParticleSystem({
      size: 0.7,
      speedDeviation: 0.15,
      tailLength: 20,
      particleSpeed: 1,
      color: 0x1133ff
    });

    var shipMesh;
    var loader = new THREE.JSONLoader();
    loader.load('Ships/enemyship.js', function(geometry, materials) {
      console.log("Ship mesh loaded.");
      var material = new THREE.MultiMaterial(materials);
      var object = new THREE.Mesh(geometry, material);
      object.visible = false;

      shipObject.add(object);
      shipObject.shipMesh = shipMesh;
      shipObject.position.set(0, 0, 0);

      shipMesh = object;
    });

    var speedLimiter = true;
    var shouldLimitSpeed = false;

    // Stores camera's rotation
    var cameraRef = new THREE.Group();
    ship.add(cameraRef);

    camera.quaternion.setFromAxisAngle(V3(1, 0, 0), 0);
    camera.position.set(0, 0, 0);
    camera.rotateOnAxis(V3(0, 1, 0), Math.PI);

    cameraRef.lookAt(V3(1, 0, 0)); // x is fwd, y is up, z is left
    var qFwd = new THREE.Quaternion().copy(cameraRef.quaternion);

    cameraRef.position.set(0, 0, 0);
    cameraRef.add(camera);

    var viewID = 0, viewCount = 3;

    console.log(qFwd);
    cameraRef.position.set(0, 0, 0);
    cameraRef.add(camera);


    function changeView(view) {
      function shipVisible(visible) {
        if (shipMesh) shipMesh.visible = visible;
      }

      if (view === 0) {
        shipVisible(false);
        camera.position.set(0, 0, 0);
      } else if (view === 1) {
        shipVisible(true);
        camera.position.set(0, 4, -8);
      } else if (view === 2) {
        shipVisible(true);
        camera.position.set(0, 7, -18);
      }

      viewID = view;
    }

    var particleSet = particleSystem.particlesys;

    shipObject.add(particleSet);
    particleSet.position.set(-0.5,1.8,0);
    particleSet.rotateOnAxis(new THREE.Vector3(1,0,0), Math.PI);

    var particleLight = new THREE.PointLight(0x66bbff, 4, 20);
    shipObject.add(particleLight);
    particleLight.position.set(0,0,-2);

    ship.changeView = changeView;

    changeView(0);

    var omegaYaw = 0.0, omegaRoll = 0.0, omegaPitch = 0.0;

    var shipRotation = new THREE.Quaternion();

    var objectives = window.location.search.indexOf('free') == -1 ?
        new Objectives(view, objectiveLabel) : null;

    ship.update = function(t, dt) {
      var currentNode = view.cursor.cursor;
      var currentBody = currentNode.data;

      var minAlt = 10 * km; // min altitude to body
      var distanceToBody = cursor.localPos.length() - currentBody.radius;

      // direction vectors
      fwd.set(1, 0, 0).applyQuaternion(shipRotation);
      up.set(0, 1, 0).applyQuaternion(shipRotation);
      right.set(0, 0, 1).applyQuaternion(shipRotation);

      back.copy(fwd).multiplyScalar(-1);
      down.copy(up).multiplyScalar(-1);
      left.copy(right).multiplyScalar(-1);

      var dv = inputAxis('f', 'r', 1e-3);

      var v = vel.length();

      function accelerate() {
        // rotate ship
        vel.copy(fwd).multiplyScalar(v);

        // move ship
        vel.addScaledVector(fwd, Math.min(10e8, v * v / 3e5 + 2e4) * dv * dt);

        v = vel.length();

        // limit speed near planets
        if (distanceToBody < 30e6 * km) {
          var speedLimit = 5e9 * Math.tanh(distanceToBody / (1e7 * km)) + 1e3 * km;
          shouldLimitSpeed = v > speedLimit && currentBody.kind != "star";
        }

        v = vel.length();

        // apply drag
        if (keyboard.pressed('z') || (speedLimiter && shouldLimitSpeed)) {
          if (v < 1000) {
            vel.add(V3().copy(fwd).multiplyScalar(-Math.min(v, 1e3 * dt)));
          } else {
            var k = shouldLimitSpeed ? 1 : 15;
            vel.multiplyScalar(Math.exp(-k * dt));
          }
        }

        v = vel.length();
        if (v > 500 * 3e8)
          vel.setLength(100 * 3e8);
      }

      function realisticAccelerate() {
        var acc = 30.0;
        dVel.copy(fwd).normalize().multiplyScalar(acc * dv * dt);
        vel.add(dVel);
      }

      accelerate();

      // collision detection against body
      if (distanceToBody < minAlt) {
        console.log("inside body");
        unitR.copy(cursor.localPos).normalize();
        var dot = unitR.dot(vel);
        if (dot < 0) {
          console.log("... moving toward body");
          var radialComponent = V3().copy(unitR).multiplyScalar(dot);
          // cancel out velocity
          vel.sub(radialComponent);
          // move outside of body by performing projection
          cursor.localPos.normalize().multiplyScalar(currentBody.radius + minAlt);
        }

        if (objectives) objectives.onCollision();
      }

      if (objectives) {
        objectives.update(t);
      }

      var kaOmega = 3.0;
      var aOmegaYaw = kaOmega * inputAxis('d', 'a', 0.1);
      var aOmegaRoll = kaOmega * inputAxis('e', 'q', 0.1);
      var aOmegaPitch = kaOmega * inputAxis('s', 'w', 0.1);

      omegaYaw += aOmegaYaw * dt;
      omegaRoll += aOmegaRoll * dt;
      omegaPitch += aOmegaPitch * dt;

      var omegaRate = 12.0, omegaPositiveRate = 2.0;

      omegaYaw *= Math.exp(-(!Math.abs(aOmegaYaw) ? omegaRate : omegaPositiveRate) * dt);
      omegaRoll *= Math.exp(-(!Math.abs(aOmegaRoll) ? omegaRate : omegaPositiveRate) * dt);
      omegaPitch *= Math.exp(-(!Math.abs(aOmegaPitch) ? omegaRate : omegaPositiveRate) * dt);

      // compute quaternion for ship incremental rotation
      var quaternion = new THREE.Quaternion();
      var qroll = quatAxisAngle(back, omegaRoll * dt);
      var qpitch = quatAxisAngle(left, omegaPitch * dt);
      var qyaw = quatAxisAngle(up, omegaYaw * dt);

      quaternion.multiplyQuaternions(qroll, qpitch);
      quaternion.multiply(qyaw);
      quaternion.multiply(shipRotation);

      // update ship orientation
      shipRotation.copy(quaternion);

      // update ship position
      var delta = V3().copy(vel).multiplyScalar(dt);
      view.moveCursor(delta);

      ship.position.set(0, 0, 0);

      if (selection) {
        var body = selection.body, orbit = body.orbit, orbitData = body.orbitData;

        var basicInfo = (
          "<strong>" + selection.body.name + "</strong><br />" +
          "Radius: " + formatDistance(selection.body.radius, 2) + "<br />" +
          "Distance: " + formatDistance(
            view.getRelativePositionTo(selection.node).length(), 5) + "<br />"
            );
        var orbitInfo = "";
        if (orbit) {
          var period = orbitData.period < 0 ?
            formatNumber(-orbitData.period / day, 3) + " days (retrograde)"
            : formatNumber(orbitData.period / day, 3) + " days";
          orbitInfo = (
              "Speed: " + formatSpeed(orbit.getVel(simTime).length(), 3) + "<br />" +
              "Eccentricity: " + formatNumber(orbitData.eccentricity, 5) + "<br />" +
              "Semi-Major Axis: " + formatDistance(orbitData.semiMajorAxis, 3) + "<br />" +
              "Orbital Period: " + period + " <br />"
              );
        }
        objectLabel.html(
            basicInfo + orbitInfo
        );
      } else {
        objectLabel.html("Click on an object.");
      }

      speedLabel.html(
          ("Auto Slow: " + (speedLimiter ? "on" : "off")) + "<br />" + formatSpeed(v, 6));

      shipObject.quaternion.multiplyQuaternions(shipRotation, qFwd);
      cameraRef.quaternion.multiplyQuaternions(shipRotation, qFwd);

      shipObject.updateMatrix();
      shipObject.updateMatrixWorld(false);

      cameraRef.updateMatrix();
      cameraRef.updateMatrixWorld(false);

      // update HUD
      HUD.update(cameraRef.getWorldRotation(), cameraRef.getWorldDirection());

      particleSet.material.opacity = Math.max(0, dv);

      particleLight.intensity = Math.max(0, dv)*3;

      particleSystem.update();
    };

    function pickBody(pos) {
      // picking a body
      raycaster.setFromCamera(pos, camera);
      var intersects = raycaster.intersectObjects(scene._targets);
      if (intersects.length) {
        for (var i = 0; i < intersects.length; i++) {
          var intersect = intersects[i];
          if (intersect.object._sphericalNodeRenderer) {
            var sr = intersect.object._sphericalNodeRenderer;
            sr.detailedInfo = true;
            console.log(sr.body.name, sr);
            if (selection === sr)
              selection = null;
            else
              selection = sr;
            break;
          }
        }
      }
    }

    function onMouseDown(event) {
      pickBody(mouse);
    }

    ship.onMouseDown = onMouseDown;

    function onKeyDown(event) {
      if (keyboard.eventMatches(event, 'v')) {
        changeView(viewID = (viewID + 1) % viewCount);
      } else if (keyboard.eventMatches(event, 'enter')) {
        pickBody(new THREE.Vector2());
      } else if (keyboard.eventMatches(event, 'g')) {
        speedLimiter = !speedLimiter;
      }
    }

    ship.onKeyDown = onKeyDown;

    return ship;
  }

  function start() {
    keyboard = new THREEx.KeyboardState();
    timer = new THREE.Clock();

    setupEvents();

    var orbitLines = new THREE.Group();
    orbitLines.name = "Orbit Lines";
    scene._orbitLines = orbitLines;
    scene.add(orbitLines);

    //system = getSampleSystem();
    system = getSolarSystem();
    systemView = new SOITreeRenderer(system, scene);

    scene.add(ship = MakeShip(systemView, camera));

    var r = system.root.radius;

    // large texture slows down integrated graphics
    var bgSphere = new THREE.SphereGeometry(camera.far - au, 32, 32);
    var bgTexture = new THREE.MeshBasicMaterial({
      side: THREE.BackSide,
      map: new THREE.TextureLoader().load("/Textures/Milky.jpg")
    });
    var bgMesh = new THREE.Mesh(bgSphere, bgTexture);

    // orientation of milky way relative to ecliptic
    bgMesh.applyMatrix(new THREE.Matrix4().makeRotationFromQuaternion(
      quatAxisAngle(new THREE.Vector3(1, 0, 0), 60 * deg)));
    scene.add(bgMesh);

    ambientLight = new THREE.AmbientLight(0x101010);
    scene.add(ambientLight);

    if( window.location.search.indexOf('free') == -1){
      myAudio = new Audio('SolarCircuit.ogg');
      myAudio.addEventListener('ended', function() {
        this.currentTime = 0;
        this.play();
      }, false);
      myAudio.play();
    }

  }

  function update() {
    var t = timer.getElapsedTime();
    var dt = t - t0;

    t0 = t;
    requestAnimationFrame(update);

    fpsStats.begin();

    var dSimTime = timeAcc * dt;
    simTime += dSimTime;
    system.time = simTime;

    ship.update(simTime, dSimTime);

    timeLabel.html(
      "Time: " + simTime.toFixed(0) + " s<br />" +
      "Time Acc: " + timeAcc.toFixed(2) + "x<br />" +
      "FOV: " + camera.fov + "&deg;");

    camera.fov = Math.max(20, Math.min(camera.fov + inputAxis('leftbracket', 'rightbracket', false), 120));

    var timeAcc1 = expAdjust(timeAcc, 5 * inputAxis('comma', 'period', 1e-2) * dt);
    timeAcc = clamp(timeAcc1, 1, 30 * day);
    if (keyboard.pressed('j')) timeAcc = 1;

    // render screen
    var left   = Math.floor(windowWidth  * 0);
    var bottom = Math.floor(windowHeight * 0);
    var width  = Math.floor(windowWidth  * 1);
    var height = Math.floor(windowHeight * 1);

    renderer.setViewport(left, bottom, width, height);
    renderer.setScissor(left, bottom, width, height);
    renderer.setScissorTest(true);
    renderer.setClearColor(0);

    camera.aspect = width / height;
    camera.updateProjectionMatrix();

    renderer.render(scene, camera);

    fpsStats.end();
  }

  setupResize();
  start();
  update();
});
